SBLOCK = 10
XMAX = 80
YMAX = 60
FACTOR = 5

function newstate()
	local t = {}
	for i=1,XMAX do
		t[i] = {}
		for j=1,YMAX do
			t[i][j] = {math.random(0, 255), math.random(0, 255), math.random(0, 255)}
		end
	end
	return t
end

function love.load()
	s1 = newstate()
	s2 = newstate()
	time = 0
end

function evolve(s1, i, j, k)
	local t = s1[i][j][k]
	local n = 1
	if s1[i-1] then
		t = t + s1[i-1][j][k]
		n = n + 1
		if s1[i-1][j-1] then
			t = t + s1[i-1][j-1][k]
			n = n + 1
		end
		if s1[i-1][j+1] then
			t = t + s1[i-1][j+1][k]
			n = n + 1
		end
	end
	if s1[i+1] then
		t = t + s1[i+1][j][k]
		n = n + 1
		if s1[i+1][j-1] then
			t = t + s1[i+1][j-1][k]
			n = n + 1
		end
		if s1[i+1][j+1] then
			t = t + s1[i+1][j+1][k]
			n = n + 1
		end
	end
	if s1[i][j-1] then
		t = t + s1[i][j-1][k]
		n = n + 1
	end
	if s1[i][j+1] then
		t = t + s1[i][j+1][k]
		n = n + 1
	end
	return math.min(255, math.max(0, t/n - s1[i][j][k%3+1]/FACTOR + s1[i][j][(k-2)%3+1]/FACTOR))
end

function step()
	for i=1,XMAX do
		for j=1,YMAX do
			for k=1,3 do
				s2[i][j][k] = evolve(s1, i, j, k)
			end
		end
	end
	s1, s2 = s2, s1
end

function love.update(dt)
	time = time + dt
	if time > 0.1 then
		step()
		time = time - 0.1
	end
end

function love.draw()
	for i=1,XMAX do
		for j=1,YMAX do
			love.graphics.setColor(s1[i][j])
			love.graphics.rectangle('fill', i * SBLOCK - SBLOCK, j * SBLOCK - SBLOCK, SBLOCK, SBLOCK)
		end
	end
	love.graphics.setColor(0, 0, 0)
	love.graphics.print("Factor (w/s): " .. FACTOR, 5, 5)
	love.graphics.print("Reset with r, close with escape", 5, 15)
end

function love.keypressed(key)
	if key == 'escape' then
		love.event.quit()
	elseif key == 'r' then
		s1 = newstate()
	elseif key == 'w' then
		FACTOR = math.min(15, FACTOR + 0.2)
	elseif key == 's' then
		FACTOR = math.max(1, FACTOR - 0.2)
	end
end
